@@ -14,15 +14,26 @@ class Install(EutesterTestCase):
         self.parser.add_argument("--nogpg",action='store_true')
         self.parser.add_argument("--nightly",action='store_true')
         self.parser.add_argument("--lvm-extents")
-        self.parser.add_argument("--root-lv")
+        self.parser.add_argument("--vnet-mode", default="MANAGED-NOVLAN")
+        self.parser.add_argument("--vnet-subnet", default="1.0.0.0")
+        self.parser.add_argument("--vnet-netmask", default="255.255.0.0")
+        self.parser.add_argument("--vnet-publicips")
+        self.parser.add_argument("--vnet_addrspernet", default="32")
+        self.parser.add_argument("--vnet_privinterface", default="br0")
+        self.parser.add_argument("--vnet_pubinterface", default="br0")
+        self.parser.add_argument("--vnet_bridge", default="br0")
+        self.parser.add_argument("--vnet-dns", default="8.8.8.8")
+        self.parser.add_argument("--root-lv", default="/dev/vg01/")
+        self.parser.add_argument("--dnsdomain")
+        self.parser.add_argument("--block-device-manager", default="das")
         if extra_args:
             for arg in extra_args:
                 self.parser.add_argument(arg)
         self.get_args()
         # Setup basic eutester object
         self.tester = Eucaops( config_file=self.args.config_file, password=self.args.password, download_creds=download_creds)
-        if not self.args.branch and not self.args.euca_url and not self.args.enterprise_url:
-            self.args.branch = self.args.upgrade_to_branch
+        #if not self.args.branch and not self.args.euca_url and not self.args.enterprise_url:
+        #    self.args.branch = self.args.upgrade_to_branch
 
     def clean_method(self):
         pass
@@ -66,6 +77,8 @@ class Install(EutesterTestCase):
         for machine in self.tester.get_component_machines("sc"):
             machine.package_manager.add_repo(self.repo_url, "eucalyptus")
         for machine in self.tester.get_component_machines("nc"):
+            if machine.distro.name is "vmware":
+                continue
             machine.package_manager.add_repo(self.repo_url, "eucalyptus")
             machine.package_manager.add_repo(self.euca2ools_url, "euca2ools")
 
@@ -81,10 +94,14 @@ class Install(EutesterTestCase):
         for machine in self.tester.get_component_machines("sc"):
             machine.package_manager.install("eucalyptus-sc")
         for machine in self.tester.get_component_machines("nc"):
+            if machine.distro.name is "vmware":
+                continue
             machine.package_manager.install("eucalyptus-nc")
 
     def start_components(self):
         for machine in self.tester.get_component_machines("nc"):
+            if machine.distro.name is "vmware":
+                continue
             machine.sys("service eucalyptus-nc start", timeout=480)
         for machine in self.tester.get_component_machines("cc"):
             machine.sys("service eucalyptus-cc start", timeout=480)
@@ -105,6 +122,8 @@ class Install(EutesterTestCase):
         for machine in self.tester.get_component_machines("cc"):
             machine.sys("service eucalyptus-cc stop", timeout=480)
         for machine in self.tester.get_component_machines("nc"):
+            if machine.distro.name is "vmware":
+                continue
             machine.sys("service eucalyptus-nc stop", timeout=480)
 
     def initialize_db(self):
@@ -115,6 +134,8 @@ class Install(EutesterTestCase):
         nc_machines = self.tester.get_component_machines("nc")
         bridge_interface = "em1"
         for nc in nc_machines:
+            if nc.distro.name is "vmware":
+                return
             nc.sys("echo 'DEVICE=br0\nBOOTPROTO=dhcp\nONBOOT=yes\nTYPE=Bridge' > /etc/sysconfig/network-scripts/ifcfg-br0")
             nc.sys("echo 'DEVICE=" + bridge_interface +"\nTYPE=Ethernet\nBRIDGE=br0' > /etc/sysconfig/network-scripts/ifcfg-" + bridge_interface)
             nc.sys("service network restart")
@@ -128,15 +149,40 @@ class Install(EutesterTestCase):
             machine.sys("lvextend " + logical_volume + " -l" + extents )
             machine.sys("resize2fs -f " + logical_volume, timeout=12000)
 
-    def wait_for_creds(self, timeout=300):
-        while timeout > 0:
+    def wait_for_creds(self, timeout=900):
+        def get_creds():
             try:
-                self.tester = Eucaops(config_file=self.args.config_file, password=self.args.password)
-                break
-            except Exception,e:
-                pass
-            timeout -= 20
-            self.tester.sleep(20)
+                self.tester = Eucaops(config_file=self.args.config_file, password=self.args.password,download_creds=False)
+                self.tester.get_credentials()
+                return True
+            except Exception:
+                return False
+        self.tester.wait_for_result(get_creds, True, timeout=timeout)
+
+    def sync_ssh_keys(self):
+        ### Sync CLC SSH key to all machines
+        clc = self.tester.get_component_machines("clc")[0]
+        clc_pub_key = clc.sys("cat ~/.ssh/id_rsa.pub")[0]
+        for machine in self.tester.get_component_machines():
+            machine.sys("echo " + clc_pub_key + " >> ~/.ssh/authorized_keys")
+
+        ### Sync CC keys to the proper NCs
+        try:
+            for cluster in ["00", "01", "02", "03"]:
+                for machine in self.tester.get_component_machines("cc" + cluster):
+                    cc_pub_key = machine.sys("cat ~/.ssh/id_rsa.pub")[0]
+                    for nc in self.tester.get_component_machines("nc" + cluster):
+                        nc.sys("echo " + cc_pub_key + " >> ~/.ssh/authorized_keys")
+        except IndexError:
+            pass
+
+    def remove_host_check(self):
+        for machine in self.tester.get_component_machines():
+            ssh_config_file = 'Host *\nStrictHostKeyChecking no\nUserKnownHostsFile=/dev/null\n'
+            #assert isinstance(machine, Machine)
+            ssh_config_file = machine.sftp.open("/root/.ssh/config", "w")
+            ssh_config_file.write(ssh_config_file)
+            ssh_config_file.close()
 
     def register_components(self):
         clcs = self.tester.get_component_machines("clc")
@@ -173,23 +219,18 @@ class Install(EutesterTestCase):
                 clcs[0].sys("euca_conf --register-sc -C storage" + str(cluster_number-1) +
                             "B -P cluster" + str(cluster_number-1) + " -H " + sc.hostname)
 
-        nodes = self.tester.get_component_machines("nc")
-        registered_nodes = {1:None, 2:None,3:None,4:None,5:None,6:None,7:None,8:None}
-        cluster_number = 1
-        for node in nodes:
-            if not registered_nodes[cluster_number]:
-                registered_nodes[cluster_number] = [node]
-                for cluster in registered_clusters[cluster_number]:
-                    cluster.sys("euca_conf --register-nodes " + node.hostname)
-                cluster_number += 1
-            else:
-                registered_nodes[cluster_number-1].append(node)
-                for cluster in registered_clusters[cluster_number-1]:
-                    cluster.sys("euca_conf --register-nodes " + node.hostname)
-        self.clc_service = self.tester.service_manager.get_enabled_clc()
+        try:
+            for cluster in ["00", "01", "02","03"]:
+                for nc in self.tester.get_component_machines("nc" + cluster):
+                    for cc in self.tester.get_component_machines("cc" + cluster):
+                        cc.sys("euca_conf --register-nodes " + nc.hostname)
+        except IndexError:
+            pass
 
     def set_block_storage_manager(self):
-        enabled_clc = self.tester.service_manager.wait_for_service(self.clc_service)
+        if not hasattr(self.tester, 'service_manager'):
+            self.tester = Eucaops(config_file=self.args.config_file, password=self.args.password)
+        enabled_clc = self.tester.service_manager.get_enabled_clc()
         self.zones = self.tester.get_zones()
         for zone in self.zones:
             ebs_manager = "overlay"
@@ -204,21 +245,50 @@ class Install(EutesterTestCase):
                             ebs_manager = "netapp"
                         if re.search("EmcVnxProvider", self.args.san_provider):
                             ebs_manager = "emc-fastsnap"
-            enabled_clc.machine.sys("source " + self.tester.credpath + "/eucarc && euca-modify-property -p " + zone + ".storage.blockstoragemanager=" + ebs_manager,code=0)
+            else:
+                ebs_manager = self.args.block_device_manager
+            self.tester.modify_property(zone + ".storage.blockstoragemanager", ebs_manager)
+            self.tester.modify_property(zone + ".storage.dasdevice", self.args.root_lv)
 
+    def set_config_option(self, machine, option, parameter):
+        sed_command = 'sed -i -e "s/^.*{0}=.*$/{0}={1}/" {2}/etc/eucalyptus/eucalyptus.conf'.format(option, parameter, self.tester.eucapath)
+        machine.sys(sed_command)
+
+    def configure_network(self):
+        for machine in self.tester.get_component_machines("cc"):
+            self.set_config_option(machine, "VNET_MODE", self.args.vnet_mode)
+            self.set_config_option(machine, "VNET_SUBNET", self.args.vnet_subnet)
+            self.set_config_option(machine, "VNET_NETMASK", self.args.vnet_netmask)
+            self.set_config_option(machine, "VNET_PUBLICIPS", self.args.vnet_publicips)
+            self.set_config_option(machine, "VNET_DNS", self.args.vnet_dns)
+            self.set_config_option(machine, "VNET_ADDRSPERNET", self.args.vnet_addrspernet)
+            self.set_config_option(machine, "VNET_PRIVINTERFACE", self.args.vnet_privinterface)
+            self.set_config_option(machine, "VNET_PUBINTERFACE", self.args.vnet_pubinterface)
+
+        for machine in self.tester.get_component_machines("nc"):
+            self.set_config_option(machine, "VNET_MODE", self.args.vnet_mode)
+            self.set_config_option(machine, "VNET_BRIDGE", self.args.vnet_bridge)
+
+    def setup_dns(self):
+        if not hasattr(self.tester, 'service_manager'):
+            self.tester = Eucaops(config_file=self.args.config_file, password=self.args.password)
+        self.tester.modify_property("bootstrap.webservices.use_dns_delegation", "true")
+        self.tester.modify_property("bootstrap.webservices.use_instance_dns", "true")
+        if self.args.dnsdomain:
+            self.tester.modify_property("system.dns.dnsdomain", self.args.dnsdomain)
+        else:
+            hostname = self.tester.service_manager.get_enabled_clc().machine.sys('hostname')[0].split(".")[0]
+            domain = hostname + ".autoqa.qa1.eucalyptus-systems.com"
+            self.tester.modify_property("system.dns.dnsdomain", domain)
 
     def clean_method(self):
         pass
 
-
     def InstallEuca(self):
-        self.add_euca_repo()
-        if self.args.enterprise_url:
-            self.add_enterprise_repo()
-        self.extend_logical_volume()
-        self.install_packages()
         self.initialize_db()
-        self.setup_bridges()
+        self.sync_ssh_keys()
+        self.remove_host_check()
+        self.configure_network()
         self.start_components()
         self.wait_for_creds()
         self.register_components()
@@ -227,12 +297,11 @@ class Install(EutesterTestCase):
 if __name__ == "__main__":
     testcase = Install()
     ### Either use the list of tests passed from config/command line to determine what subset of tests to run
-    list = testcase.args.tests or ["InstallEuca"]
+    test_list = testcase.args.tests or ["InstallEuca"]
     ### Convert test suite methods to EutesterUnitTest objects
     unit_list = [ ]
-    for test in list:
+    for test in test_list:
         unit_list.append( testcase.create_testunit_by_name(test) )
         ### Run the EutesterUnitTest objects
-
     result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
     exit(result)
