@@ -37,7 +37,8 @@ import os
 from eucaops import Eucaops
 from eutester import euinstance
 from eutester.eutestcase import EutesterTestCase
-
+from eucaops import ec2ops
+from eutester.eutestcase import TestColor
 
 class TestZone():
     def __init__(self, partition):
@@ -65,19 +66,23 @@ class EbsTestSuite(EutesterTestCase):
                  tester=None, 
                  zone=None, 
                  config_file='../input/2b_tested.lst', 
-                 password="foobar", 
+                 password="foobar",
+                 user_data=None,
                  inst_pass=None,
                  credpath=None, 
                  volumes=None, 
                  keypair=None, 
                  group=None, 
                  emi=None,
+                 waitconnect=30,
+                 wait_on_progress=20,
                  root_device_type='instance-store',
                  vmtype='c1.medium',
                  eof=1):
         
         self.args = args
         self.setuptestcase(name)
+        self.wait_on_progress = wait_on_progress
         if tester is None:
             self.tester = Eucaops( config_file=config_file,password=password,credpath=credpath)
         else:
@@ -90,11 +95,11 @@ class EbsTestSuite(EutesterTestCase):
             self.image = self.tester.get_emi(emi=emi)
         else:
             self.image = self.tester.get_emi(root_device_type=root_device_type, not_location='windows')
-        
         self.vmtype = vmtype
-        self.zone = None    
+        self.zone = None
+        self.waitconnect=int(waitconnect)
         self.zonelist = []
-            
+        self.user_data = user_data
         #create some zone objects and append them to the zonelist
         if self.zone:
             self.zone = TestZone(zone)
@@ -218,6 +223,7 @@ class EbsTestSuite(EutesterTestCase):
                                                 group=group,
                                                 username=username,
                                                 password=inst_pass,
+                                                user_data=self.user_data,
                                                 type=vmtype,
                                                 zone=zone,
                                                 min=count,
@@ -250,12 +256,7 @@ class EbsTestSuite(EutesterTestCase):
             for instance in zone.instances:
                 instance.terminate_and_verify(verify_vols=True,timeout=timeout)
                 zone.instances.remove(instance)
-                
-               
-                
-                
-        
-    
+
     def negative_attach_in_use_volume_in_zones(self,zonelist=None,timeout=360):
         """
         Description:
@@ -316,6 +317,13 @@ class EbsTestSuite(EutesterTestCase):
                         instance = zone.instances[i]
                         try:
                             instance.attach_euvolume(volume,timeout=timeout, overwrite=overwrite)
+                        except ec2ops.VolumeStateException, vse:
+                            self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
+                                        testcolor=TestColor.get_canned_color('failred'))
+                            time.sleep(10)
+                            self.debug('Monitoring volume post VolumeStateException...')
+                            volume.eutest_attached_status = None
+                            self.tester.monitor_euvolumes_to_status([volume],status='in-use',attached_status='attached',timeout=60)
                         except Exception, e:
                             self.debug("attach_all_vols_to_instances_in_zones failed to attach volume")
                             raise e
@@ -495,13 +503,14 @@ class EbsTestSuite(EutesterTestCase):
                 
         
         
-    def create_snapshots_all_vols_in_zone(self, zonelist=None, volstate="all", wait_on_progress=20):
+    def create_snapshots_all_vols_in_zone(self, zonelist=None, volstate="all", wait_on_progress=None):
         """
         Description:
                     Attempts to iterate through each zone in zonelist, and create a snapshot from each volume
                     in the zone's volume list who's state matches volstate
                     
         """
+        wait_on_progress = wait_on_progress or self.wait_on_progress
         zonelist = zonelist or self.zonelist
         if not zonelist:
             raise Exception("Zone list was empty")
@@ -511,7 +520,7 @@ class EbsTestSuite(EutesterTestCase):
             for volume in zone.volumes:
                 volume.update()
                 if volstate == "all" or volume.status == volstate:
-                    self.snaps.append(self.tester.create_snapshot_from_volume(volume, description="ebstest", wait_on_progress=20))
+                    self.snaps.append(self.tester.create_snapshot_from_volume(volume, description="ebstest", wait_on_progress=wait_on_progress))
         #self.endsuccess()
         
         
@@ -578,7 +587,18 @@ class EbsTestSuite(EutesterTestCase):
                         if i > len(zone.instances)-1:
                             i = 0
                         instance = zone.instances[i]
-                        instance.attach_euvolume(vol, timeout=timeout)
+                        try:
+                            instance.attach_euvolume(vol, timeout=timeout)
+                        except ec2ops.VolumeStateException, vse:
+                            self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
+                                        testcolor=TestColor.get_canned_color('failred'))
+                            time.sleep(10)
+                            self.debug('Monitoring volume post VolumeStateException...')
+                            vol.eutest_attached_status = None
+                            self.tester.monitor_euvolumes_to_status([vol],status='in-use',attached_status='attached',timeout=60)
+                        except Exception, e:
+                            self.debug("Failed to attach volume: " + str(vol.id) + "to instance:" + str())
+                            raise e
                         instance.md5_attached_euvolume(vol, timepergig=timepergig)
                         if vol.md5 != snap.eutest_volume_md5:
                             self.debug("snap:"+str(snap.eutest_volume_md5)+" vs vol:"+str(vol.md5))
@@ -614,7 +634,8 @@ class EbsTestSuite(EutesterTestCase):
                                                 delay=0, 
                                                 tpg=300,
                                                 delete_to=120,
-                                                poll_progress=60):
+                                                poll_progress=60,
+                                                attach_timeout=360):
         """
         Description:
                    Attempts to create a 'count' number of snapshots consecutively with a delay of 'delay'
@@ -651,7 +672,18 @@ class EbsTestSuite(EutesterTestCase):
                 self.tester.print_euvolume_list(vols)
                 self.status("Attempting to attach new vols from new snapshots to instance:"+str(instance.id)+" to verify md5s...")
                 for newvol in vols:
-                    instance.attach_volume(newvol)
+                    try:
+                        instance.attach_euvolume(newvol,timeout=attach_timeout)
+                    except ec2ops.VolumeStateException, vse:
+                        self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
+                                    testcolor=TestColor.get_canned_color('failred'))
+                        time.sleep(10)
+                        self.debug('Monitoring volume post VolumeStateException...')
+                        newvol.eutest_attached_status = None
+                        self.tester.monitor_euvolumes_to_status([newvol],status='in-use',attached_status='attached',timeout=60)
+                    except Exception, e:
+                        self.debug("Failed to attach volume: " + str(newvol.id) + "to instance:" + str())
+                        raise e
                     if vol.md5 != newvol.md5:
                         raise Exception("New volume's md5:"+str(newvol.md5)+" !=  original volume md5:"+str(vol.md5))
                     else:
@@ -682,7 +714,8 @@ class EbsTestSuite(EutesterTestCase):
                                                             delay=0, 
                                                             tpg=300,
                                                             delete_to=120,
-                                                            poll_progress=60):
+                                                            poll_progress=60,
+                                                            attach_timeout=360):
         """
         Description:
                    Attempts to create a 'count' number of volumes from a given snapshot consecutively with a delay of 'delay'
@@ -724,7 +757,18 @@ class EbsTestSuite(EutesterTestCase):
                 self.status("Attempting to attach new vols from new snapshots to instance:"+str(instance.id)+" to verify md5s...")      
                 for newvol in vols:
                     if newvol.zone == zone.name:
-                        instance.attach_volume(newvol)
+                        try:
+                            instance.attach_euvolume(newvol, timeout=attach_timeout)
+                        except ec2ops.VolumeStateException, vse:
+                            self.status("This is a temp work around for testing, this is to avoid bug euca-5297"+str(vse),
+                                    testcolor=TestColor.get_canned_color('failred'))
+                            time.sleep(10)
+                            self.debug('Monitoring volume post VolumeStateException...')
+                            newvol.eutest_attached_status = None
+                            self.tester.monitor_euvolumes_to_status([newvol],status='in-use',attached_status='attached',timeout=60)
+                        except Exception, e:
+                            self.debug("Failed to attach volume: " + str(newvol.id) + "to instance:" + str())
+                            raise e
                         #Compare MD5 sum to original volume
                         if str(origmd5).rstrip().lstrip() != str(newvol.md5).rstrip().lstrip():
                             raise Exception("New volume's md5:'"+str(newvol.md5)+"' !=  original volume md5:'"+str(origmd5)+"'")
@@ -808,15 +852,36 @@ class EbsTestSuite(EutesterTestCase):
             self.run_test_case_list(testlist)
         else:
             return testlist
-                   
-    
-    
-                
-        
+
+
+    def expand_volume_size(self, zonelist=None, volsperzone=1, size=1):
+        """
+        Description:
+                    Intention of this test is to verify creation of volume(s) from a snapshot and expanding
+                    the size of the volume
+        """
+        zonelist = zonelist or self.zonelist
+        if not zonelist:
+            raise Exception("Zone list was empty")
+        for testzone in zonelist:
+            vols = self.tester.create_volumes(testzone, size=size, count=volsperzone)
+            testzone.volumes.extend(vols)
+            snapshots = []
+            for volume in vols:
+                snapshots.append(self.tester.create_snapshot_from_volume(volume))
+            larger_volumes = []
+            for snaphot in snapshots:
+                larger_volumes.append(self.tester.create_volume(testzone, snapshot=snaphot, size=size+1))
+            for volume in larger_volumes:
+                assert volume.size > size
+
+
     def ebs_basic_test_suite(self, run=True):  
         testlist = [] 
         #create first round of volumes
         testlist.append(self.create_testunit_from_method(self.create_vols_per_zone, eof=True))
+        #create volumes that have their sizes expanded from their original snapshots
+        testlist.append(self.create_testunit_from_method(self.expand_volume_size, eof=True))
         #launch instances to interact with ebs volumes
         testlist.append(self.create_testunit_from_method(self.create_test_instances_for_zones, eof=True))
         #attach first round of volumes
@@ -830,7 +895,7 @@ class EbsTestSuite(EutesterTestCase):
         #attach second round of volumes
         testlist.append(self.create_testunit_from_method(self.attach_all_avail_vols_to_instances_in_zones, overwrite=True))
         #reboot instances and confirm volumes remain attached
-        testlist.append(self.create_testunit_from_method(self.reboot_instances_in_zone_verify_volumes))
+        testlist.append(self.create_testunit_from_method(self.reboot_instances_in_zone_verify_volumes, waitconnect=self.waitconnect))
         #detach 1 volume leave the 2nd attached
         testlist.append(self.create_testunit_from_method(self.detach_volumes_in_zones))
         #attempt to create volumes from snaps, attach and verify md5 in same zone it was created in
@@ -913,9 +978,18 @@ class EbsTestSuite(EutesterTestCase):
         if zones is None or zones == []:
             zones = self.zones
     def clean_method(self):
+        """
+        Definition:
+        Attempts to clean up test artifacts created during this test
+        """
+
         self.clean_created_resources(zonelist=self.zonelist, timeout=360)
     
     def clean_created_resources(self, zonelist=None, timeout=360):
+        """
+        Definition:
+        Attempts to clean up test artifacts created during this test
+        """
         self.tester.cleanup_artifacts()
         #self.terminate_test_instances_for_zones(zonelist=zonelist, timeout=timeout)
         #self.delete_volumes_in_zones(zonelist=zonelist, timeout=timeout)
