@@ -214,9 +214,9 @@ class WindowsTests(EutesterTestCase):
             if re.search(match,image.location):
                 retlist.append(image)
         return retlist
-        
-    
-        
+
+
+
     def update_proxy_instance_data(self, win_instance=None, instance_password=None ):
         if self.proxy is None:
             return
@@ -346,6 +346,7 @@ class WindowsTests(EutesterTestCase):
                                       destpath = None, 
                                       inter_bundle_timeout = None, 
                                       upload_timeout = None,
+                                      virtualization_type = None,
                                       wget_user = None,
                                       wget_password = None,
                                       time_per_gig = None,
@@ -363,12 +364,13 @@ class WindowsTests(EutesterTestCase):
                                            interbundle_timeout = (inter_bundle_timeout or self.inter_bundle_timeout), 
                                            upload_timeout = (upload_timeout or self.upload_timeout),
                                            destpath = (destpath or self.destpath),
+                                           virtualization_type=virtualization_type,
                                            wget_user = (wget_user), 
                                            wget_password = (wget_password),   
                                            time_per_gig = (time_per_gig or self.time_per_gig) )
         
     def create_windows_emi_from_file(self,
-                                     fpath,
+                                     image_file_path,
                                      component = None, 
                                      component_credpath = None,
                                      bucketname = None, 
@@ -382,7 +384,8 @@ class WindowsTests(EutesterTestCase):
         Work is done on a given machine and requires euca2ools present on that machine. 
         Returns the emi of the registered image
         '''
-        return self.iu.create_emi_from_url( 
+        return self.iu.create_emi_from_url(
+                                           url = None,
                                            component = (component or self.component), 
                                            bucketname = (bucketname or self.bucketname), 
                                            component_credpath = (component_credpath or self.component_credpath), 
@@ -390,7 +393,7 @@ class WindowsTests(EutesterTestCase):
                                            interbundle_timeout = (inter_bundle_timeout or self.inter_bundle_timeout), 
                                            upload_timeout = (upload_timeout or self.upload_timeout),
                                            destpath = (destpath or self.destpath),
-                                           filepath = fpath,
+                                           filepath = image_file_path,
                                            time_per_gig = (time_per_gig or self.time_per_gig) )
         
     def test_run_windows_emi(self,
@@ -422,7 +425,12 @@ class WindowsTests(EutesterTestCase):
                                  group = group or self.group, 
                                  type = type, 
                                  zone=zone, 
-                                 min=min, max=max, user_data=user_data, private_addressing=private_addressing, is_reachable=False, timeout=timeout)
+                                 min=min,
+                                 max=max,
+                                 user_data=user_data,
+                                 private_addressing=private_addressing,
+                                 is_reachable=False,
+                                 timeout=timeout)
         
         
         self.instance = res.instances[0]
@@ -577,9 +585,7 @@ class WindowsTests(EutesterTestCase):
                 return dev
         raise Exception('Instance:'+str(instance.id)+", no free devs. Has "+str(count)+" devices in use by ebs already. max:"+str(max))    
         
-                
-                
-        
+
            
     def test_attach_single_volume(self, instance=None, dev=None):
         instance = instance or self.instance
